@@ -7,7 +7,7 @@
 #' @param encoding File encoding (default: \code{"UTF-8"}), \code{"auto"} for auto-detect of Unicode encoding
 #'
 #' @return A Formant object represents formants as a function of time.
-#' @return   [ref: Praat help, http://www.fon.hum.uva.nl/praat/manual/Formant.html]
+#' @return   [ref: Praat help, https://www.fon.hum.uva.nl/praat/manual/Formant.html]
 #' @return   \code{f$xmin} ... start time (seconds)
 #' @return   \code{f$xmax} ... end time (seconds)
 #' @return   \code{f$nx}   ... number of frames
@@ -21,7 +21,7 @@
 #' @return      \code{f$frame[[1]]$frequency} ... vector of formant frequencies (in Hz)
 #' @return      \code{f$frame[[1]]$bandwidth} ... vector of formant bandwidths (in Hz)
 #' @export
-#' @seealso \code{\link{pitch.read}}, \code{\link{pt.read}}, \code{\link{tg.read}}, \code{\link{it.read}}, \code{\link{col.read}}
+#' @seealso \code{\link{formant.write}}, \code{\link{formant.plot}}, \code{\link{formant.cut}}, \code{\link{formant.getPointIndexNearestTime}}, \code{\link{pitch.read}}, \code{\link{pt.read}}, \code{\link{tg.read}}, \code{\link{it.read}}, \code{\link{col.read}}
 #'
 #' @examples
 #' \dontrun{
@@ -134,8 +134,19 @@ formant.read_lines <- function(flines, find = 1, collection = FALSE) {
                     }
                     iline <- iline + 1
 
-                    frequency[If] <- as.numeric(substr(strTrim(flines[iline]), 13, nchar(strTrim(flines[iline])))); iline <- iline + 1
-                    bandwidth[If] <- as.numeric(substr(strTrim(flines[iline]), 13, nchar(strTrim(flines[iline])))); iline <- iline + 1
+                    nmbr <- substr(strTrim(flines[iline]), 13, nchar(strTrim(flines[iline]))); iline <- iline + 1
+                    if (nmbr != "--undefined--") {
+                        frequency[If] <- as.numeric(nmbr)
+                    } else {
+                        frequency[If] <- NA
+                    }
+
+                    nmbr <- substr(strTrim(flines[iline]), 13, nchar(strTrim(flines[iline]))); iline <- iline + 1
+                    if (nmbr != "--undefined--") {
+                        bandwidth[If] <- as.numeric(nmbr)
+                    } else {
+                        bandwidth[If] <- NA
+                    }
                 }
 
                 frame[[I]] <- list(intensity = intensity, nFormants = nFormants,
@@ -162,8 +173,19 @@ formant.read_lines <- function(flines, find = 1, collection = FALSE) {
                 bandwidth <- numeric(nFormants)
 
                 for (If in seqM(1, nFormants)) {
-                    frequency[If] <- as.numeric(flines[iline]); iline <- iline + 1
-                    bandwidth[If] <- as.numeric(flines[iline]); iline <- iline + 1
+                    nmbr <- flines[iline]; iline <- iline + 1
+                    if (nmbr != "--undefined--") {
+                        frequency[If] <- as.numeric(nmbr)
+                    } else {
+                        frequency[If] <- NA
+                    }
+
+                    nmbr <- flines[iline]; iline <- iline + 1
+                    if (nmbr != "--undefined--") {
+                        bandwidth[If] <- as.numeric(nmbr)
+                    } else {
+                        bandwidth[If] <- NA
+                    }
                 }
 
                 frame[[I]] <- list(intensity = intensity, nFormants = nFormants,
@@ -645,12 +667,28 @@ formant.write0 <- function(formant, fileNameFormant, format = "short", fid = NUL
 
         for (I in seqM(1, formant$frame[[N]]$nFormants)) {
             if (format == "short") {
-                wrLine(as.character(formant$frame[[N]]$frequency[I], -15), fid)
-                wrLine(as.character(formant$frame[[N]]$bandwidth[I], -15), fid)
+                if (!is.na(formant$frame[[N]]$frequency[I])) {
+                    wrLine(as.character(round2(formant$frame[[N]]$frequency[I], -15)), fid)
+                } else {
+                    wrLine("--undefined--", fid)
+                }
+                if (!is.na(formant$frame[[N]]$bandwidth[I])) {
+                    wrLine(as.character(round2(formant$frame[[N]]$bandwidth[I], -15)), fid)
+                } else {
+                    wrLine("--undefined--", fid)
+                }
             } else if (format == "text") {
                 wrLine(paste0("            formant [", as.character(I), "]:"), fid, collection)
-                wrLine(paste0("                frequency = ", as.character(formant$frame[[N]]$frequency[I], -15), " "), fid, collection)
-                wrLine(paste0("                bandwidth = ", as.character(formant$frame[[N]]$bandwidth[I], -15), " "), fid, collection)
+                if (!is.na(formant$frame[[N]]$frequency[I])) {
+                    wrLine(paste0("                frequency = ", as.character(round2(formant$frame[[N]]$frequency[I], -15)), " "), fid, collection)
+                } else {
+                    wrLine(paste0("                frequency = --undefined-- "), fid, collection)
+                }
+                if (!is.na(formant$frame[[N]]$bandwidth[I])) {
+                    wrLine(paste0("                bandwidth = ", as.character(round2(formant$frame[[N]]$bandwidth[I], -15)), " "), fid, collection)
+                } else {
+                    wrLine(paste0("                bandwidth = --undefined-- "), fid, collection)
+                }
             }
         }
     }
@@ -659,3 +697,118 @@ formant.write0 <- function(formant, fileNameFormant, format = "short", fid = NUL
         close(fid)
     }
 }
+
+
+#' formant.getPointIndexHigherThanTime
+#'
+#' Returns index of frame which is nearest the given time from right, i.e.
+#' \code{time} <= frameTime.
+#'
+#' @param formant Formant object
+#' @param time time which is going to be found in frames
+#'
+#' @return integer
+#' @export
+#' @seealso \code{\link{formant.getPointIndexNearestTime}}, \code{\link{formant.getPointIndexLowerThanTime}}
+#'
+#' @examples
+#' formant <- formant.sample()
+#' formant.getPointIndexHigherThanTime(formant, 0.5)
+formant.getPointIndexHigherThanTime <- function(formant, time) {
+    if (!isNum(time)) {
+        stop("Time must be a number.")
+    }
+
+    ind <- NA
+
+    npoints <- length(formant$t)
+    for (I in seqM(1, npoints)) {
+        if (time <= formant$t[I]) {
+            ind <- I
+            break
+        }
+    }
+
+
+    return(ind)
+}
+
+
+
+
+#' formant.getPointIndexLowerThanTime
+#'
+#' Returns index of frame which is nearest the given time from left, i.e.
+#' frameTime <= \code{time}.
+#'
+#' @param formant Formant object
+#' @param time time which is going to be found in frames
+#'
+#' @return integer
+#' @export
+#' @seealso \code{\link{formant.getPointIndexNearestTime}}, \code{\link{formant.getPointIndexHigherThanTime}}
+#'
+#' @examples
+#' formant <- formant.sample()
+#' formant.getPointIndexLowerThanTime(formant, 0.5)
+formant.getPointIndexLowerThanTime <- function(formant, time) {
+    if (!isNum(time)) {
+        stop("Time must be a number.")
+    }
+
+    ind <- NA
+
+    npoints <- length(formant$t)
+    for (I in seqM(npoints, 1, by = -1)) {
+        if (time >= formant$t[I]) {
+            ind <- I
+            break
+        }
+    }
+
+
+    return(ind)
+}
+
+
+
+
+#' formant.getPointIndexNearestTime
+#'
+#' Returns index of frame which is nearest the given \code{time} (from both sides).
+#'
+#' @param formant Formant object
+#' @param time time which is going to be found in frames
+#'
+#' @return integer
+#' @export
+#' @seealso \code{\link{formant.getPointIndexLowerThanTime}}, \code{\link{formant.getPointIndexHigherThanTime}}
+#'
+#' @examples
+#' formant <- formant.sample()
+#' formant.getPointIndexNearestTime(formant, 0.5)
+formant.getPointIndexNearestTime <- function(formant, time) {
+    if (!isNum(time)) {
+        stop("Time must be a number.")
+    }
+
+    ind <- NA
+
+    npoints <- length(formant$t)
+    minDist <- Inf
+    minInd <- NA
+
+    for (I in seqM(1, npoints)) {
+        dist <- abs(formant$t[I] - time)
+        if (dist < minDist) {
+            minDist <- dist
+            minInd <- I
+        }
+    }
+
+    ind <- minInd
+
+
+    return(ind)
+}
+
